Package: NMF
Type: Package
Title: Algorithms and Framework for Nonnegative Matrix Factorization
        (NMF)
Version: 0.21.0
Date: 2018-02-18
Author: Renaud Gaujoux, Cathal Seoighe
Maintainer: Nicolas Sauwen <nicolas.sauwen@openanalytics.eu>
Description: Provides a framework to perform Non-negative Matrix
    Factorization (NMF). The package implements a set of already published algorithms
    and seeding methods, and provides a framework to test, develop and plug
    new/custom algorithms. Most of the built-in algorithms have been optimized
    in C++, and the main interface function provides an easy way of performing
    parallel computations on multicore machines.
License: GPL (>= 2)
URL: http://renozao.github.io/NMF
LazyLoad: yes
VignetteBuilder: knitr
Depends: R (>= 3.0.0), methods, utils, pkgmaker (>= 0.20), registry,
        rngtools (>= 1.2.3), cluster
Imports: graphics, stats, stringr (>= 1.0.0), digest, grid, grDevices,
        gridBase, colorspace, RColorBrewer, foreach, doParallel,
        ggplot2, reshape2
Suggests: fastICA, doMPI, bigmemory (>= 4.2), synchronicity(>= 1.3.2),
        corpcor, xtable, devtools, knitr, bibtex, RUnit, mail, Biobase
Collate: 'colorcode.R' 'options.R' 'grid.R' 'atracks.R' 'aheatmap.R'
        'algorithmic.R' 'nmf-package.R' 'rmatrix.R' 'utils.R'
        'versions.R' 'NMF-class.R' 'transforms.R' 'Bioc-layer.R'
        'NMFstd-class.R' 'NMFOffset-class.R' 'heatmaps.R'
        'NMFns-class.R' 'nmfModel.R' 'fixed-terms.R' 'NMFfit-class.R'
        'NMFSet-class.R' 'NMFStrategy-class.R' 'registry.R'
        'NMFSeed-class.R' 'NMFStrategyFunction-class.R'
        'NMFStrategyIterative-class.R' 'NMFplots.R'
        'registry-algorithms.R' 'algorithms-base.R' 'algorithms-lnmf.R'
        'algorithms-lsnmf.R' 'algorithms-pe-nmf.R' 'algorithms-siNMF.R'
        'algorithms-snmf.R' 'data.R' 'extractFeatures.R' 'parallel.R'
        'registry-seed.R' 'nmf.R' 'rnmf.R' 'run.R' 'seed-base.R'
        'seed-ica.R' 'seed-nndsvd.R' 'setNMFClass.R' 'simulation.R'
        'tests.R'
Packaged: 2018-03-06 09:53:11 UTC; nsauwen
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2018-03-06 16:35:36 UTC
RoxygenNote: 6.0.1
Built: R 3.3.2; x86_64-w64-mingw32; 2018-03-10 12:26:11 UTC; windows
Archs: x64
