# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,NMF)
S3method(nmfFormals,NMFStrategy)
S3method(nmfFormals,NMFStrategyFunction)
S3method(nmfFormals,NMFStrategyIterative)
S3method(nmfFormals,character)
S3method(plot,NMF.consensus)
S3method(plot,NMF.rank)
S3method(print,fcnnls)
S3method(print,foreach_backend)
S3method(profplot,default)
S3method(register,doMPI_backend)
S3method(register,doParallel_backend)
S3method(register,foreach_backend)
S3method(scale,NMF)
S3method(silhouette,NMF)
S3method(silhouette,NMFfitX)
S3method(summary,NMF.rank)
S3method(t,NMF)
S3method(t,NMFstd)
export(".basis<-")
export(".coef<-")
export("algorithm<-")
export("basis<-")
export("basisnames<-")
export("coef<-")
export("fit<-")
export("name<-")
export("niter<-")
export("objective<-")
export("residuals<-")
export("seeding<-")
export(.atrack)
export(.basis)
export(.coef)
export(.fcnnls)
export(.getRNG)
export(ForeachBackend)
export(NMFSeed)
export(NMFStop)
export(NMFStrategy)
export(aheatmap)
export(algorithm)
export(atrack)
export(basis)
export(basiscor)
export(basismap)
export(basisnames)
export(bterms)
export(canFit)
export(coef)
export(coefficients)
export(coefmap)
export(compare)
export(connectivity)
export(consensus)
export(consensushc)
export(consensusmap)
export(cophcor)
export(cterms)
export(deviance)
export(dispersion)
export(entropy)
export(evar)
export(existsNMFMethod)
export(existsNMFSeed)
export(extractFeatures)
export(fcnnls)
export(featureScore)
export(fit)
export(fitted)
export(gVariable)
export(getDoBackend)
export(getDoParHosts)
export(getDoParNHosts)
export(getNMFMethod)
export(getNMFSeed)
export(getRNG1)
export(hasBasis)
export(hasCoef)
export(hasTrack)
export(hostfile)
export(ibasis)
export(ibterms)
export(icoef)
export(icterms)
export(is.empty.nmf)
export(is.mixed)
export(is.nmf)
export(is.partial.nmf)
export(isNMFfit)
export(iterms)
export(loadings)
export(logs)
export(match_atrack)
export(metaHeatmap)
export(minfit)
export(misc)
export(modelname)
export(name)
export(nbasis)
export(nbterms)
export(ncterms)
export(neq.constraints.inplace)
export(niter)
export(nmf)
export(nmf.equal)
export(nmf.getOption)
export(nmf.options)
export(nmf.printOptions)
export(nmf.resetOptions)
export(nmf.stop.connectivity)
export(nmf.stop.iteration)
export(nmf.stop.stationary)
export(nmf.stop.threshold)
export(nmfAlgorithm)
export(nmfApply)
export(nmfArgs)
export(nmfCheck)
export(nmfDistance)
export(nmfEstimateRank)
export(nmfFormals)
export(nmfModel)
export(nmfModels)
export(nmfObject)
export(nmfRegisterAlgorithm)
export(nmfReport)
export(nmfSeed)
export(nmfWrapper)
export(nmf_update.KL.h)
export(nmf_update.KL.h_R)
export(nmf_update.KL.w)
export(nmf_update.KL.w_R)
export(nmf_update.brunet)
export(nmf_update.brunet_R)
export(nmf_update.euclidean.h)
export(nmf_update.euclidean.h_R)
export(nmf_update.euclidean.w)
export(nmf_update.euclidean.w_R)
export(nmf_update.euclidean_offset.h)
export(nmf_update.euclidean_offset.w)
export(nmf_update.lee)
export(nmf_update.lee_R)
export(nmf_update.ns)
export(nmf_update.ns_R)
export(nmf_update.offset)
export(nmf_update.offset_R)
export(nneg)
export(nrun)
export(nterms)
export(objective)
export(offset)
export(pmax.inplace)
export(posneg)
export(predict)
export(profcor)
export(profplot)
export(purity)
export(randomize)
export(register)
export(removeNMFMethod)
export(removeNMFSeed)
export(residuals)
export(rmatrix)
export(rnmf)
export(rss)
export(run)
export(runtime)
export(runtime.all)
export(scoef)
export(seed)
export(seeding)
export(seqtime)
export(setDoBackend)
export(setNMFMethod)
export(setNMFSeed)
export(smoothing)
export(sparseness)
export(staticVar)
export(str_args)
export(summary)
export(syntheticNMF)
export(trackError)
export(ts_eval)
export(ts_tempfile)
export(which.best)
exportClasses(NMF)
exportClasses(NMFList)
exportClasses(NMFOffset)
exportClasses(NMFfit)
exportClasses(NMFfitX)
exportClasses(NMFfitX1)
exportClasses(NMFfitXn)
exportClasses(NMFns)
exportClasses(NMFstd)
exportMethods("$")
exportMethods("$<-")
exportMethods("[")
exportMethods("basisnames<-")
exportMethods("dimnames<-")
exportMethods("objective<-")
exportMethods(.DollarNames)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(fitted)
exportMethods(objective)
exportMethods(plot)
exportMethods(rposneg)
exportMethods(show)
exportPattern("^featureNames")
exportPattern("^metagenes")
exportPattern("^metaprofiles")
exportPattern("^nmeta")
exportPattern("^sampleNames")
import(RColorBrewer)
import(cluster)
import(digest)
import(doParallel)
import(foreach)
import(ggplot2)
import(grDevices)
import(graphics)
import(grid)
import(gridBase)
import(methods)
import(pkgmaker)
import(registry)
import(reshape2)
import(rngtools)
import(stats)
import(stringr)
importFrom(colorspace,sequential_hcl)
importFrom(utils,.DollarNames)
useDynLib(NMF, .registration = TRUE)