
\documentclass[10pt]{article}
%\VignetteDepends{knitr}
%\VignetteIndexEntry{NMF-unitTests}
%\VignetteCompiler{knitr}
%\VignetteEngine{knitr::knitr}
\usepackage{vmargin}
\setmargrb{0.75in}{0.75in}{0.75in}{0.75in}

<<setup, include=FALSE>>=
pkg <- 'NMF'
require( pkg, character.only=TRUE )
prettyVersion <- packageDescription(pkg)$Version
prettyDate <- format(Sys.Date(), '%B %e, %Y')
authors <- packageDescription(pkg)$Author
@

\usepackage[colorlinks]{hyperref}
\author{\Sexpr{authors}}
\title{\texttt{\Sexpr{pkg}}: Unit testing results\footnote{Vignette computed  on Tue Mar  4 13:14:49 2014}}
\date{\texttt{\Sexpr{pkg}} version \Sexpr{prettyVersion} as of \Sexpr{prettyDate}}
\begin{document}
\maketitle

\section{Details}
\begin{verbatim}

RUNIT TEST PROTOCOL -- Tue Mar  4 13:14:49 2014 
*********************************************** 
Number of test functions: 81 
Number of deactivated test functions: 5 
Number of errors: 0 
Number of failures: 0 

 
1 Test Suite : 
package:NMF - 81 test functions, 0 errors, 0 failures



Details 
*************************** 
Test Suite: package:NMF 
Test function regexp: ^test. 
Test file regexp: ^runit.*.[rR]$ 
Involved directory: 
/tmp/Rpkglib_26ff65706d7b/NMF/tests 
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.aheatmap.R 
test.mfrow: (1 checks) ... OK (0.69 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.algorithms.r 
test.brunet: (16 checks) ... OK (4.22 seconds)
test.cversions.brunet: (1 checks) ... OK (0.87 seconds)
test.cversions.lee: (1 checks) ... OK (1.03 seconds)
test.cversions.lnmf : DEACTIVATED, Algorithm 'lnmf' is not fully working.
test.cversions.ns: (1 checks) ... OK (1.11 seconds)
test.cversions.offset: (1 checks) ... OK (2.19 seconds)
test.frobenius: (16 checks) ... OK (5.38 seconds)
test.KL: (16 checks) ... OK (4.23 seconds)
test.lee: (16 checks) ... OK (3.72 seconds)
test.lnmf : DEACTIVATED, Algorithm 'lnmf' is not fully working.
test.ns: (16 checks) ... OK (3.78 seconds)
test.offset: (16 checks) ... OK (3.97 seconds)
test.port_brunet : DEACTIVATED, Package RcppOctave not available.
test.snmf: (44 checks) ... OK (9.39 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.bioc.r 
test.access: (26 checks) ... OK (0.02 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.distance.r 
test.KL: (9 checks) ... OK (1.44 seconds)
test.rss: (4 checks) ... OK (0.06 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.interface.r 
test.compare: (5 checks) ... OK (9.3 seconds)
test.nmf.callback: (22 checks) ... OK (13.81 seconds)
test.nmf.custom: (87 checks) ... OK (1.5 seconds)
test.nmf.default: (8 checks) ... OK (0.23 seconds)
test.nmf.dots: (11 checks) ... OK (0.84 seconds)
test.nmf.method: (36 checks) ... OK (10.79 seconds)
test.nmf.model: (22 checks) ... OK (0.62 seconds)
test.nmfModel.formula: (12 checks) ... OK (0.92 seconds)
test.nmf.multirank: (10 checks) ... OK (5.68 seconds)
test.nmf.options: (18 checks) ... OK (5.44 seconds)
test.nmf.parameters: (53 checks) ... OK (0.81 seconds)
test.nmf.seed.argument: (149 checks) ... OK (9.4 seconds)
test.nmf.seed.equivalent: (11 checks) ... OK (10.82 seconds)
test.nmf.seed.fault: (4 checks) ... OK (2.9 seconds)
test.nmf.seed.repro: (8 checks) ... OK (14.43 seconds)
test.nmf.stop: (6 checks) ... OK (2.17 seconds)
test.parallel: (6 checks) ... OK (12.28 seconds)
test.registry: (9 checks) ... OK (0.13 seconds)
test.seed: (33 checks) ... OK (0.13 seconds)
test.summary: (3 checks) ... OK (2.86 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.NMFclass.r 
test.basis: (2 checks) ... OK (0.01 seconds)
test.class.NMFns: (65 checks) ... OK (0.03 seconds)
test.class.NMFstd: (28 checks) ... OK (0.01 seconds)
test.coef: (2 checks) ... OK (0 seconds)
test.connectivity: (3 checks) ... OK (0.01 seconds)
test.deviance: (9 checks) ... OK (0.02 seconds)
test.dimensions: (4 checks) ... OK (0 seconds)
test.dimnames: (173 checks) ... OK (0.08 seconds)
test.entropy: (5 checks) ... OK (0 seconds)
test.misc: (9 checks) ... OK (0 seconds)
test.nmfModel: (363 checks) ... OK (0.11 seconds)
test.NMF.rnmf: (93 checks) ... OK (1.5 seconds)
test.predict: (10 checks) ... OK (0.01 seconds)
test.purity: (5 checks) ... OK (0 seconds)
test.sparseness: (13 checks) ... OK (0.01 seconds)
test.subset: (50 checks) ... OK (0.05 seconds)
test.syntheticNMF: (5 checks) ... OK (1.62 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.NMFfit-class.r 
test.deviance: (0 checks) ... OK (0 seconds)
test.isNMFfit: (6 checks) ... OK (3.59 seconds)
test.niter: (4 checks) ... OK (1.1 seconds)
test.nmf.equal: (304 checks) ... OK (1.87 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.NMFSet.r 
test.fit: (9 checks) ... OK (3.85 seconds)
test.interface: (6 checks) ... OK (1.86 seconds)
test.join.multipleAndSingleRunsMethods: (2 checks) ... OK (3.32 seconds)
test.join.multipleRuns: (2 checks) ... OK (4.6 seconds)
test.join.singleRuns: (7 checks) ... OK (0.7 seconds)
test.multipleruns: (2 checks) ... OK (1.7 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.NMFStrategy-class.r 
test.accessors: (26 checks) ... OK (0.06 seconds)
test.constructor: (16 checks) ... OK (0.06 seconds)
test.constructorMethod: (3 checks) ... OK (0.04 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.options.r 
test.nmf.getOption: (2 checks) ... OK (0 seconds)
test.nmf.options: (9 checks) ... OK (0 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.parallel.r 
test.ForeachBackend: (4 checks) ... OK (1.37 seconds)
test.gVariable: (11 checks) ... OK (10.16 seconds)
test.nmf: (59 checks) ... OK (41.99 seconds)
test.setupBackend: (4 checks) ... OK (0.05 seconds)
test.shared_memory_doMC: (12 checks) ... OK (6.1 seconds)
test.shared_memory_doMPI : DEACTIVATED, NMF shared memory feature does not currently work with doMPI.
test.shared_memory_doParallel: (12 checks) ... OK (10.88 seconds)
test.shared_memory_doSEQ: (11 checks) ... OK (6.46 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.seed.r 
test.nndsvd: (10 checks) ... OK (0.04 seconds)
test.none: (6 checks) ... OK (0.03 seconds)
test.random: (26 checks) ... OK (0.13 seconds)
test.restore : DEACTIVATED, The option 'restore.seed' is deprecated. Related tests are now in test.seed.effect
test.seed.effect: (15 checks) ... OK (16.37 seconds)
test.zzz.all: (297 checks) ... OK (16.42 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_26ff65706d7b/NMF/tests/runit.utils.r 
test.nmfWrapper: (16 checks) ... OK (0.85 seconds)
test.rmatrix: (16 checks) ... OK (0.01 seconds)

Total execution time
***************************
   user  system elapsed 
253.753  31.645 293.297 

\end{verbatim}

\section*{Session Information}
\begin{itemize}\raggedright
  \item R Under development (unstable) (2014-03-02 r65102), \verb|x86_64-unknown-linux-gnu|
  \item Locale: \verb|LC_CTYPE=en_US.UTF-8|, \verb|LC_NUMERIC=C|, \verb|LC_TIME=en_US.UTF-8|, \verb|LC_COLLATE=en_US.UTF-8|, \verb|LC_MONETARY=en_US.UTF-8|, \verb|LC_MESSAGES=en_US.UTF-8|, \verb|LC_PAPER=en_US.UTF-8|, \verb|LC_NAME=C|, \verb|LC_ADDRESS=C|, \verb|LC_TELEPHONE=C|, \verb|LC_MEASUREMENT=en_US.UTF-8|, \verb|LC_IDENTIFICATION=C|
  \item Base packages: base, datasets, graphics, grDevices, methods,
    parallel, stats, utils
  \item Other packages: BH~1.51.0-4, bigmemory~4.4.6,
    bigmemory.sri~0.1.2, Biobase~2.22.0, BiocGenerics~0.8.0,
    cluster~1.14.4, doMPI~0.2, doParallel~1.0.8, fastICA~1.2-0,
    foreach~1.4.1, iterators~1.0.6, NMF~0.20.2, pkgmaker~0.17.4,
    RColorBrewer~1.0-5, Rcpp~0.11.0, registry~0.2, Rmpi~0.6-3,
    rngtools~1.2.3, RUnit~0.4.26, stringr~0.6.2, synchronicity~1.1.2
  \item Loaded via a namespace (and not attached): codetools~0.2-8,
    colorspace~1.2-4, compiler~3.1.0, dichromat~2.0-0, digest~0.6.4,
    ggplot2~0.9.3.1, grid~3.1.0, gridBase~0.4-7, gtable~0.1.2,
    labeling~0.2, MASS~7.3-29, munsell~0.4.2, plyr~1.8.1, proto~0.3-10,
    reshape2~1.2.2, scales~0.2.3, tools~3.1.0, xtable~1.7-1
\end{itemize}

\end{document}

